@@ -3,10 +3,11 @@ Stream Novelty Detection for River (StreamNDR) is a Python library for online no
 StreamNDR aims to enable novelty detection in data streams for Python by implementing various 
 algorithms that have been proposed in the literature.
 """
-from . import model, utils
+from . import metrics, model, utils
 from .__version__ import __version__ # noqa: F401
 
 __all__ = [
+    "metrics",
     "model",
     "utils",
 ]
\ No newline at end of file
